@@ -90,6 +90,7 @@ struct lxc_arguments {
 
 	/* auto-start */
 	int all;
+	int ignore_auto;
 	int list;
 	char *groups;
 
@@ -121,6 +122,8 @@ extern int lxc_arguments_parse(struct lxc_arguments *args,
 
 extern int lxc_arguments_str_to_int(struct lxc_arguments *args, const char *str);
 
+extern const char *lxc_strerror(int errnum);
+
 #define lxc_error(arg, fmt, args...) if (!(arg)->quiet)			\
 	fprintf(stderr, "%s: " fmt "\n", (arg)->progname,  ## args)
 
