@@ -22,32 +22,28 @@
  */
 #include "config.h"
 
-#ifndef _caps_h
-#define _caps_h
+#ifndef __LXC_CAPS_H
+#define __LXC_CAPS_H
 
 #if HAVE_SYS_CAPABILITY_H
-extern int lxc_caps_reset(void);
 extern int lxc_caps_down(void);
 extern int lxc_caps_up(void);
 extern int lxc_caps_init(void);
 
 extern int lxc_caps_last_cap(void);
 #else
-static inline int lxc_caps_reset(void) {
-        return 0;
-}
 static inline int lxc_caps_down(void) {
-        return 0;
+	return 0;
 }
 static inline int lxc_caps_up(void) {
-        return 0;
+	return 0;
 }
 static inline int lxc_caps_init(void) {
-        return 0;
+	return 0;
 }
 
 static inline int lxc_caps_last_cap(void) {
-        return 0;
+	return 0;
 }
 #endif
 
