@@ -67,6 +67,8 @@ static int config_idmap(const char *, const char *, struct lxc_conf *);
 static int config_loglevel(const char *, const char *, struct lxc_conf *);
 static int config_logfile(const char *, const char *, struct lxc_conf *);
 static int config_mount(const char *, const char *, struct lxc_conf *);
+static int config_mount_auto(const char *, const char *, struct lxc_conf *);
+static int config_fstab(const char *, const char *, struct lxc_conf *);
 static int config_rootfs(const char *, const char *, struct lxc_conf *);
 static int config_rootfs_mount(const char *, const char *, struct lxc_conf *);
 static int config_rootfs_options(const char *, const char *, struct lxc_conf *);
@@ -118,7 +120,9 @@ static struct lxc_config_t config[] = {
 	{ "lxc.id_map",               config_idmap                },
 	{ "lxc.loglevel",             config_loglevel             },
 	{ "lxc.logfile",              config_logfile              },
-	{ "lxc.mount",                config_mount                },
+	{ "lxc.mount.entry",          config_mount                },
+	{ "lxc.mount.auto",           config_mount_auto           },
+	{ "lxc.mount",                config_fstab                },
 	{ "lxc.rootfs.mount",         config_rootfs_mount         },
 	{ "lxc.rootfs.options",       config_rootfs_options       },
 	{ "lxc.rootfs",               config_rootfs               },
@@ -1123,10 +1127,10 @@ static int config_group(const char *key, const char *value,
 	/* in case several groups are specified in a single line
 	 * split these groups in a single element for the list */
 	for (groupptr = groups;;groupptr = NULL) {
-                token = strtok_r(groupptr, " \t", &sptr);
-                if (!token) {
+		token = strtok_r(groupptr, " \t", &sptr);
+		if (!token) {
 			ret = 0;
-                        break;
+			break;
 		}
 
 		grouplist = malloc(sizeof(*grouplist));
@@ -1143,7 +1147,7 @@ static int config_group(const char *key, const char *value,
 		}
 
 		lxc_list_add_tail(&lxc_conf->groups, grouplist);
-        }
+	}
 
 	free(groups);
 
@@ -1485,7 +1489,7 @@ static int config_mount_auto(const char *key, const char *value,
 		{ "cgroup-full:mixed",  LXC_AUTO_CGROUP_MASK,    LXC_AUTO_CGROUP_FULL_MIXED  },
 		{ "cgroup-full:ro",     LXC_AUTO_CGROUP_MASK,    LXC_AUTO_CGROUP_FULL_RO     },
 		{ "cgroup-full:rw",     LXC_AUTO_CGROUP_MASK,    LXC_AUTO_CGROUP_FULL_RW     },
-		/* NB: For adding anything that ist just a single on/off, but has
+		/* NB: For adding anything that is just a single on/off, but has
 		 *     no options: keep mask and flag identical and just define the
 		 *     enum value as an unused bit so far
 		 */
@@ -1506,10 +1510,10 @@ static int config_mount_auto(const char *key, const char *value,
 	}
 
 	for (autoptr = autos; ; autoptr = NULL) {
-                token = strtok_r(autoptr, " \t", &sptr);
-                if (!token) {
+		token = strtok_r(autoptr, " \t", &sptr);
+		if (!token) {
 			ret = 0;
-                        break;
+			break;
 		}
 
 		for (i = 0; allowed_auto_mounts[i].token; i++) {
@@ -1524,47 +1528,19 @@ static int config_mount_auto(const char *key, const char *value,
 
 		lxc_conf->auto_mounts &= ~allowed_auto_mounts[i].mask;
 		lxc_conf->auto_mounts |= allowed_auto_mounts[i].flag;
-        }
+	}
 
 	free(autos);
 
 	return ret;
 }
 
-/*
- * TODO
- * This fn is handling lxc.mount, lxc.mount.entry, and lxc.mount.auto.
- * It should probably be split into 3 separate functions indexed by
- * the config[] entries at top.
- */
 static int config_mount(const char *key, const char *value,
 			struct lxc_conf *lxc_conf)
 {
-	char *fstab_token = "lxc.mount";
-	char *token = "lxc.mount.entry";
-	char *auto_token = "lxc.mount.auto";
-	char *subkey;
 	char *mntelem;
 	struct lxc_list *mntlist;
 
-	subkey = strstr(key, token);
-
-	if (!subkey) {
-		subkey = strstr(key, auto_token);
-
-		if (!subkey) {
-			subkey = strstr(key, fstab_token);
-
-			if (!subkey)
-				return -1;
-
-			return config_fstab(key, value, lxc_conf);
-		}
-
-		return config_mount_auto(key, value, lxc_conf);
-	}
-
-	/* At this point we definitely have key = lxc.mount.entry */
 	if (!value || strlen(value) == 0)
 		return lxc_clear_mount_entries(lxc_conf);
 
@@ -1603,10 +1579,10 @@ static int config_cap_keep(const char *key, const char *value,
 	/* in case several capability keep is specified in a single line
 	 * split these caps in a single element for the list */
 	for (keepptr = keepcaps;;keepptr = NULL) {
-                token = strtok_r(keepptr, " \t", &sptr);
-                if (!token) {
+		token = strtok_r(keepptr, " \t", &sptr);
+		if (!token) {
 			ret = 0;
-                        break;
+			break;
 		}
 
 		if (!strcmp(token, "none"))
@@ -1626,7 +1602,7 @@ static int config_cap_keep(const char *key, const char *value,
 		}
 
 		lxc_list_add_tail(&lxc_conf->keepcaps, keeplist);
-        }
+	}
 
 	free(keepcaps);
 
@@ -1652,10 +1628,10 @@ static int config_cap_drop(const char *key, const char *value,
 	/* in case several capability drop is specified in a single line
 	 * split these caps in a single element for the list */
 	for (dropptr = dropcaps;;dropptr = NULL) {
-                token = strtok_r(dropptr, " \t", &sptr);
-                if (!token) {
+		token = strtok_r(dropptr, " \t", &sptr);
+		if (!token) {
 			ret = 0;
-                        break;
+			break;
 		}
 
 		droplist = malloc(sizeof(*droplist));
@@ -1672,7 +1648,7 @@ static int config_cap_drop(const char *key, const char *value,
 		}
 
 		lxc_list_add_tail(&lxc_conf->caps, droplist);
-        }
+	}
 
 	free(dropcaps);
 
@@ -2029,7 +2005,7 @@ int lxc_fill_elevated_privileges(char *flaglist, int *flags)
 		/* for the sake of backward compatibility, drop all privileges
 		   if none is specified */
 		for (i = 0; all_privs[i].token; i++) {
-	                *flags |= all_privs[i].flag;
+			*flags |= all_privs[i].flag;
 		}
 		return 0;
 	}
