@@ -26,38 +26,55 @@
 #include <sys/stat.h>
 #include <sys/mount.h>
 #include <sys/apparmor.h>
+#include <sys/vfs.h>
 
 #include "log.h"
 #include "lsm/lsm.h"
+#include "conf.h"
 
 lxc_log_define(lxc_apparmor, lxc);
 
 /* set by lsm_apparmor_drv_init if true */
 static int aa_enabled = 0;
 
+static int mount_features_enabled = 0;
+
 #define AA_DEF_PROFILE "lxc-container-default"
 #define AA_MOUNT_RESTR "/sys/kernel/security/apparmor/features/mount/mask"
 #define AA_ENABLED_FILE "/sys/module/apparmor/parameters/enabled"
-#define AA_UNCHANGED "unchanged"
+
+static bool check_mount_feature_enabled(void)
+{
+	return mount_features_enabled == 1;
+}
+
+static void load_mount_features_enabled(void)
+{
+	struct stat statbuf;
+	int ret;
+
+	ret = stat(AA_MOUNT_RESTR, &statbuf);
+	if (ret == 0)
+		mount_features_enabled = 1;
+}
 
 /* aa_getcon is not working right now.  Use our hand-rolled version below */
 static int apparmor_enabled(void)
 {
-	struct stat statbuf;
 	FILE *fin;
 	char e;
 	int ret;
 
-	ret = stat(AA_MOUNT_RESTR, &statbuf);
-	if (ret != 0)
-		return 0;
 	fin = fopen(AA_ENABLED_FILE, "r");
 	if (!fin)
 		return 0;
 	ret = fscanf(fin, "%c", &e);
 	fclose(fin);
-	if (ret == 1 && e == 'Y')
+	if (ret == 1 && e == 'Y') {
+		load_mount_features_enabled();
 		return 1;
+	}
+
 	return 0;
 }
 
@@ -109,33 +126,12 @@ again:
 	return buf;
 }
 
-/*
- * Probably makes sense to reorganize these to only read
- * the label once
- */
-static bool apparmor_am_unconfined(void)
+static int apparmor_am_unconfined(void)
 {
 	char *p = apparmor_process_label_get(getpid());
-	bool ret = false;
+	int ret = 0;
 	if (!p || strcmp(p, "unconfined") == 0)
-		ret = true;
-	free(p);
-	return ret;
-}
-
-/* aa stacking is not yet supported */
-static bool aa_stacking_supported(void) {
-	return false;
-}
-
-/* are we in a confined container? */
-static bool in_aa_confined_container(void) {
-	char *p = apparmor_process_label_get(getpid());
-	bool ret = false;
-	if (p && strcmp(p, "/usr/bin/lxc-start") != 0 && strcmp(p, "unconfined") != 0) {
-		INFO("Already apparmor-confined under %s", p);
-		ret = true;
-	}
+		ret = 1;
 	free(p);
 	return ret;
 }
@@ -144,6 +140,7 @@ static bool in_aa_confined_container(void) {
  * apparmor_process_label_set: Set AppArmor process profile
  *
  * @label   : the profile to set
+ * @conf    : the container configuration to use @label is NULL
  * @default : use the default profile if label is NULL
  * @on_exec : this is ignored.  Apparmor profile will be changed immediately
  *
@@ -151,37 +148,32 @@ static bool in_aa_confined_container(void) {
  *
  * Notes: This relies on /proc being available.
  */
-static int apparmor_process_label_set(const char *label, int use_default,
-				      int on_exec)
+static int apparmor_process_label_set(const char *inlabel, struct lxc_conf *conf,
+				      int use_default, int on_exec)
 {
+	const char *label = inlabel ? inlabel : conf->lsm_aa_profile;
+
 	if (!aa_enabled)
 		return 0;
 
-	/* user may request that we just ignore apparmor */
-	if (label && strcmp(label, AA_UNCHANGED) == 0) {
-		INFO("apparmor profile unchanged per user request");
-		return 0;
-	}
-
-	/*
-	 * If we are already confined and no profile was requested,
-	 * then default to unchanged
-	 */
-	if (in_aa_confined_container() && !aa_stacking_supported()) {
-		if (label) {
-			ERROR("already apparmor confined, but new label requested.");
-			return -1;
-		}
-		return 0;
-	}
-
 	if (!label) {
 		if (use_default)
 			label = AA_DEF_PROFILE;
 		else
-			return 0;
+			label = "unconfined";
 	}
 
+	if (!check_mount_feature_enabled() && strcmp(label, "unconfined") != 0) {
+		WARN("Incomplete AppArmor support in your kernel");
+		if (!conf->lsm_aa_allow_incomplete) {
+			ERROR("If you really want to start this container, set");
+			ERROR("lxc.aa_allow_incomplete = 1");
+			ERROR("in your container configuration file");
+			return -1;
+		}
+	}
+
+
 	if (strcmp(label, "unconfined") == 0 && apparmor_am_unconfined()) {
 		INFO("apparmor profile unchanged");
 		return 0;
